'use strict';

const domain = require('domain');
const { constructFromEncodedString } = require('@codefresh-io/authenticated-entity');
const { registerCorrelation } = require('./correlation');
const { REQUEST_ID_HEADER, AUTHENTICATED_ENTITY_HEADER } = require('./headers');

module.exports = {
    newDomainMiddleware() {
        return (res, req, next) => {
            const dom = domain.create();

            const requestId = res.headers[REQUEST_ID_HEADER.toLowerCase()];
            const rawAuthenticatedEntity = res.headers[AUTHENTICATED_ENTITY_HEADER.toLowerCase()];

            let authenticatedEntity = null;
            if (rawAuthenticatedEntity) {
                authenticatedEntity = constructFromEncodedString(rawAuthenticatedEntity);
            }

            dom.add(req);
            dom.add(res);

            dom.run(() => {
                try {
                    registerCorrelation(authenticatedEntity, requestId);
                    setImmediate(next);
                } catch (err) {
                    setImmediate(next, err);
                }
            });
        };
    }
};
