'use strict';

const RETRY_STATUS_CODES = [502, 503, 504];

function envVarBasedStrategy(envVar) {
    return {
        rertyOnEnvironmentVariableStrategy: () => {
            return process.env[envVar];
        },
        excludeOnEnvironmentVariableStrategy: () => {
            return !process.env[envVar];
        },
    };

}

function specifcErrorCodesStrategy(err, response = {}) {
    return RETRY_STATUS_CODES.includes(response.statusCode);
}


function compositionStrategy(...strategies) {

    return (err, response, body, options) => {
        return strategies.some((strategy) => {
            return strategy(err, response, body, options);
        });
    };
}

module.exports = {
    envVarBasedStrategy,
    specifcErrorCodesStrategy,
    compositionStrategy
};
